/*
 * Image plugin to VDR (C++)
 *
 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "data-image.h"
#include "data.h"

cFileSources ImageSources;

// --- cImage -------------------------------------------------------------

cImage::cImage(const char *Name, cFileSource * Source)
{
    name = fullname = 0;
    source = 0;
    source = Source;
    if(Name)
	Set(Name);
    source->Block();
}

cImage::~cImage()
{
    Clear();
    source->Unblock();
}

void cImage::Set(const char *Name)
{
    Clear();
    name = strdup(Name);
    name = source->BuildName(name);
}

void cImage::Clear(void)
{
    free(name);
    name = 0;
    free(fullname);
    fullname = 0;
}

// -- cSlideShow --------------------------------------------------------------

cSlideShow::cSlideShow(const char *Name, cFileSource * Source)
{
    name = basename = fullname = 0;
    source = Source;
    source->Block();
    if(Name)
	Set(Name);
}

cSlideShow::~cSlideShow()
{
    free(name);
    free(basename);
    free(fullname);
    source->Unblock();
}

void cSlideShow::Set(const char *Name)
{
    free(name);
    name = strdup(Name);
    free(basename);
    basename = strdup(name);
    free(fullname);
    fullname = source->BuildName(name);
}

// -- cInstantPlayList ------------------------------------------------------

cISlideShow::cISlideShow(cDirItem * Item):cSlideShow(Item->Name ? Item->Name : "SlideShow", Item->Source),
item(Item->Source, Item->Subdir, Item->Name,
     Item->Type)
{
}

bool cISlideShow::Load(void)
{
    bool res = false;
    Clear();
    switch (item.Type)
	{
	case itFile:
	    dsyslog("SlideShow: file %s\n", item.Name);
	    DoItem(item.Source, item.Subdir, item.Name);
	    res = true;
	    break;
	case itDir:
	    {
		dsyslog("SlideShow: dir name:%s\n", item.Name);
		char *path = item.Path();
		res =
		    ScanDir(item.Source, path, stFile, item.Source->Include(),
			    0, true);
		free(path);
		break;
	    }
	case itBase:
	    dsyslog("SlideShow: base\n");
	    res =
		ScanDir(item.Source, 0, stFile, item.Source->Include(), 0,
			true);
	    break;
	default:
	    break;
	}
    return res;
}

void cISlideShow::DoItem(cFileSource * src, const char *subdir,
			 const char *name)
{
    char *path = (char *)name;
    if(subdir)
	path = AddPath(subdir, name);
    Add(new cImage(path, src));
    if(subdir)
	free(path);
}

// -- cSlideShows --------------------------------------------------------------

bool cSlideShows::Load(cFileSource * Source)
{
    Clear();
    return ScanDir(Source, 0, stFile, "*", 0, false);
}

void cSlideShows::DoItem(cFileSource * src, const char *subdir,
			 const char *name)
{
    char *path = (char *)name;
    if(subdir)
	path = AddPath(subdir, name);
    Add(new cSlideShow(path, src));
    if(subdir)
	free(path);
}
