/*
 * Image plugin to VDR (C++)
 *
 * (C) 2003 Kai Tobias Burwieck <kai@burwieck.net>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#include <ctype.h>
#include <dirent.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <unistd.h>
#include <typeinfo>

#include <vdr/status.h>
#include <vdr/font.h>
#ifdef HAVE_BEAUTYPATCH
#include <vdr/fontsym.h>
#endif


#include "menu.h"
#include "data-image.h"
#include "menu-image.h"
#include "player-image.h"
#include "i18n.h"



// --- cMenuSetupImage --------------------------------------------------------

cMenuSetupImage::cMenuSetupImage(void)
{
    data = ImageSetup;
    SetSection(tr("Image"));
    Add(new	cMenuEditBoolItem(tr("Setup.Image$SlideShow ?"), 		&data.SlideShow,  tr("no"), tr("yes")));
    Add(new	cMenuEditIntItem (tr("Setup.Image$Slide duration (sec)"), 	&data.SSsec, 1, 300));
    Add(new	cMenuEditStrItem (tr("Setup.Image$Temp Directory"), 		data.TempDir, 128, "abcdefghijklmopqrstuvwxyz/-"));
    Add(new	cMenuEditIntItem (tr("Setup.Image$Remote Layout"), 		&data.RemoteLayout, 0, 3));
    Add(new	cMenuEditBoolItem(tr("Setup.Image$Show Date"), 			&data.ShowDate, tr("no"), tr("yes")));
    Add(new	cMenuEditBoolItem(tr("Setup.Image$Auto Repeat"), 		&data.AutoRepeat, tr("no"), tr("yes")));
    Add(new	cMenuEditBoolItem(tr("Setup.Image$Use DeviceStillPicture"), 	&data.UseDeviceStillPicture, tr("no"), tr("yes")));
}

void cMenuSetupImage::Store(void)
{
    ImageSetup = data;
    SetupStore("SlideShow", 			ImageSetup.SlideShow);
    SetupStore("SSsec", 				ImageSetup.SSsec);
    SetupStore("TempDir", 				ImageSetup.TempDir);
    SetupStore("RemoteLayout", 			ImageSetup.RemoteLayout);
    SetupStore("ShowDate", 				ImageSetup.ShowDate);
    SetupStore("AutoRepeat", 			ImageSetup.AutoRepeat);
    SetupStore("UseDeviceStillPicture", ImageSetup.UseDeviceStillPicture);
}

// --- cImageControl ---------------------------------------------------------

class cImageControl:public cImagePlayerControl {
  private:
    static cSlideShow *sshow;
    bool visible, modeOnly, play, DoShowMode;
    int lastaktnum;
    bool jumpactive, jumphide, jumpmode;
    int jumpval;
    int lastCurrent, lastTotal;
    int lastSlideMode;
    time_t timeoutShow;
    bool zoomactive;
    int zoom_val;
    int zoom_xoff;
    int zoom_yoff;
    int zoom_xres;
    int zoom_yres;
    int zoom_xcenter;
    int zoom_ycenter;
    char zoom_command[20];
    void DisplayAtBottom(const char *s);
    void ShowProgress(void);
    void ShowMode(void);
    void ShowTimed(int Seconds = 0);
    void Jump(void);
    void JumpProcess(eKeys Key);
    void JumpDisplay(void);
    void PictureZoom(void);
    void Zoom(void);
    void ZoomProcess(eKeys Key);
  public:
    cImageControl(void);
    virtual ~ cImageControl();
    virtual eOSState ProcessKey(eKeys Key);
    virtual void Hide(void);
    virtual void Show(void) {	ShowTimed();   }
	bool Visible(void) {	return visible;    }
    static bool SetSlideShow(cSlideShow * SShow);
};

cSlideShow *cImageControl::sshow = 0;

cImageControl::cImageControl(void):cImagePlayerControl(sshow)
{
    visible = modeOnly = cstarted = false;
    ss = ImageSetup.SlideShow;
    sshow = 0;
    timeoutShow = 0;
    lastTotal = ImageCount();

    // Support fr Zoom
    zoomactive = 0;
    zoom_val = 2;
    zoom_xoff = 0;
    zoom_yoff = 0;
    zoom_xres = 0;
    zoom_yres = 0;
    zoom_xcenter = 0;
    zoom_ycenter = 0;
}

cImageControl::~cImageControl()
{
    Hide();
    Stop();
    delete sshow;
}

bool cImageControl::SetSlideShow(cSlideShow * SShow)
{
    delete sshow;
    sshow = SShow;
    if(sshow)
	{
	    cControl::Launch(new cImageControl);
	}
    return false;
}

void cImageControl::ShowMode(void)
{
    if(zoomactive)
	{
	    if(!visible)
		{
		    Interface->Open(0, -1);
		    visible = modeOnly = true;
		}
	    if(visible && modeOnly)
		{
		    char buf[52];
		    eDvbFont OldFont;
		    if(!cstarted)
			snprintf(buf, sizeof(buf), "Zoom %dx", zoom_val);
		    else
			snprintf(buf, sizeof(buf), tr("Convert..."));
		    OldFont = Interface->SetFont(fontFix);
		    DisplayAtBottom(buf);
		    Interface->SetFont(OldFont);
		}
	    return;
	}

    if(!jumpactive)
	{
	    if(!visible)
		{
		    Interface->Open(0, -1);
		    visible = modeOnly = true;
		}
	    if(visible && modeOnly)
		{
		    char buf[52];
		    eDvbFont OldFont;
		    const char *Mode;
		    if(ss && !cstarted)
			Mode = " > ";
		    else if(!ss && !cstarted)
			Mode = " || ";
		    else
			Mode = tr("Convert...");
		    strn0cpy(buf, Mode, sizeof(buf));
		    OldFont = Interface->SetFont(fontFix);
		    DisplayAtBottom(buf);
		    Interface->SetFont(OldFont);
		}
	}

    if(jumpactive)
	{
	    if(cstarted)
		{
		    char buf[52];
		    eDvbFont OldFont;
		    const char *Mode;

		    Mode = tr("Convert...");
		    strn0cpy(buf, Mode, sizeof(buf));
		    OldFont = Interface->SetFont(fontFix);
		    DisplayAtBottom(buf);
		    Interface->SetFont(OldFont);
	    } else
		{
		    char buf[52];
		    eDvbFont OldFont;
		    const char *Mode;

		    Mode = tr("Jumpmenu: Select picture via key 1..9!");
		    strn0cpy(buf, Mode, sizeof(buf));
		    OldFont = Interface->SetFont(fontFix);
		    DisplayAtBottom(buf);
		    Interface->SetFont(OldFont);
		}
	}
}

void cImageControl::ShowTimed(int Seconds)
{
    if(modeOnly)
	Hide();
    if(!visible)
	{
	    ShowProgress();
	    timeoutShow = Seconds > 0 ? time(0) + Seconds : 0;
	}
}

void cImageControl::Hide(void)
{
    if(visible)
	{
	    Interface->Close();
	    visible = modeOnly = false;
	}
}

void cImageControl::DisplayAtBottom(const char *s)
{
    const int p = modeOnly ? 0 : 2;
    if(s)
	{
	    const int d = max(Width() - cOsd::WidthInCells(s), 0) / 2;
	    if(modeOnly)
		Interface->Fill(0, p, Interface->Width(), 1, clrTransparent);
	    Interface->Write(d, p, s);
    } else
	Interface->Fill(12, p, Width() - 22, 1, clrBackground);
}

void cImageControl::ShowProgress(void)
{
    bool flush = false;
    if(!visible)
	{
	    Interface->Open(Setup.OSDwidth, -3);
	    visible = true;
	}
    Interface->Clear();
    if(filename)
	{
	    char str1[72];
	    if(cstarted)
		{
		    snprintf(str1, sizeof(str1), "... %s", tr("Convert..."));
	    } else
		{
		    int n = strlen(filename);
		    if(n > Width())
			{
			    n = n - Width() + 4;
			    if(n < 0)
				n = 0;
			    snprintf(str1, sizeof(str1), "... %s",
				     filename + n);
		    } else
			{
			    snprintf(str1, sizeof(str1), "%s", filename);
			}
		}
	    Interface->Write(0, 0, str1);
	    flush = true;
	    cProgressBar ProgressBar(Width() * cOsd::CellWidth(),
				     cOsd::LineHeight(), aktnum, maxnum);
	    Interface->SetBitmap(0, cOsd::LineHeight(), ProgressBar);

	    if(jumpactive)
		{
#if 0
		    char buf[64];
		    const char *j = tr("Jump: "), u = jumpmode ? '%' : ' ';
		    const int p = modeOnly ? 0 : 2;

		    if(!jumpval)
			snprintf(buf, sizeof(buf), "%s- %c", j, u);
		    else
			snprintf(buf, sizeof(buf), "%s%d- %c", j, jumpval, u);
		    const int d = max(Width() - cOsd::WidthInCells(buf), 0) / 2;
		    Interface->Write(d, p, buf);
#else
		    char buf[64];
		    const int p = modeOnly ? 0 : 2;

		    snprintf(buf, sizeof(buf),
			     tr("Jumpmenu: Select picture via key 1..9!"));
		    const int d = max(Width() - cOsd::WidthInCells(buf), 0) / 2;
		    Interface->Write(d, p, buf);
#endif
	    } else
		{
		    char str3[72];
		    char ifss[10];
		    snprintf(ifss, sizeof(ifss), "| %s | %d/%d", "S", t3,
			     cstarted ? 0 : ImageSetup.SSsec);
		    snprintf(str3, sizeof(str3), "(%d/%d) %s", aktnum, maxnum,
			     ss ? ifss : "");
		    Interface->Write(0, 2, str3);

		    int fSource = 0;
		    struct stat stFile;
		    struct tm *timestr;
		    if(!cstarted
		       && ImageSetup.ShowDate
		       && filename
		       && ((fSource = open(filename, O_RDONLY)) > 0))
			{
			    if(!fstat(fSource, &stFile)
			       && (timestr = localtime(&(stFile.st_mtime))))
				{
				    snprintf(str3, sizeof(str3),
					     "(%2.2d.%2.2d.%2.2d-%2.2d:%2.2d)",
					     (int)timestr->tm_mday,
					     (int)timestr->tm_mon + 1,
					     (int)timestr->tm_year % 100,
					     (int)timestr->tm_hour,
					     (int)timestr->tm_min);
			    } else
				{
				    snprintf(str3, sizeof(str3),
					     "                ");
				}
			    Interface->Write(Width() - 16, 2, str3);
			    close(fSource);
		    } else
			{
			    snprintf(str3, sizeof(str3), "                ");
			    Interface->Write(Width() - 16, 2, str3);
			}
		}
	    flush = true;
	    lastaktnum = aktnum;
	    if(flush)
		Interface->Flush();
	    ShowMode();
	}
}

void cImageControl::JumpDisplay(void)
{
#if 0
    char buf[64];
    const char *j = tr("Jump: "), u = jumpmode ? '%' : ' ';
    if(!jumpval)
	snprintf(buf, sizeof(buf), "%s- %c", j, u);
    else
	snprintf(buf, sizeof(buf), "%s%d- %c", j, jumpval, u);
    DisplayAtBottom(buf);
#else
    char buf[64];
    snprintf(buf, sizeof(buf), tr("Jumpmenu: Select picture via key 1..9!"));
    DisplayAtBottom(buf);
#endif
}

void cImageControl::JumpProcess(eKeys Key)
{
#if 0
    const int n = Key - k0;
    switch (Key)
	{
	case k0...k9:
	    {
		const int max = jumpmode ? 100 : lastTotal;
		if(jumpval * 10 + n <= max)
		    jumpval = jumpval * 10 + n;
		JumpDisplay();
	    }
	    break;
	case kBlue:
	    jumpmode = !jumpmode;
	    jumpval = 0;
	    DisplayAtBottom(0);
	    JumpDisplay();
	    break;
	case kPlay:
	case kUp:
	    if(!jumpmode)
		{
		    GotoImage(jumpval);
		    jumpactive = false;
		    ss = lastSlideMode;
		}
	    break;
	case kFastRew:
	case kLeft:
	    if(!jumpmode)
		{
		    PrevImage(jumpval);
		    jumpactive = false;
		    ss = lastSlideMode;
		}

	case kFastFwd:
	case kRight:
	    if(!jumpmode)
		{
		    NextImage(jumpval);
		    jumpactive = false;
		    ss = lastSlideMode;
		}
	    break;
	default:
	    jumpactive = false;
	    ss = lastSlideMode;
	    break;
	}

    if(!jumpactive)
	{
	    if(jumphide)
		Hide();
	    else
		DisplayAtBottom(0);
	}
#else
    const int n = Key - k0;
    switch (Key)
	{
	case k1...k9:
	    if(!jumpmode)
		{
		    NextImage(n - 1);
		    jumpactive = false;
		    ss = lastSlideMode;
		}
	    break;
	case kUp:
	case kRight:
	    if(!jumpmode)
		{
		    ConvertJump(9);
		}
	    break;
	case kDown:
	case kLeft:
	    if(!jumpmode)
		{
		    ConvertJump(-9);
		}
	    break;
	default:
	    NextImage(0);
	    jumpactive = false;
	    ss = lastSlideMode;
	    break;
	}

    if(!jumpactive)
	{
	    if(jumphide)
		Hide();
	    else
		DisplayAtBottom(0);
	}
#endif
}

void cImageControl::Jump(void)
{
#if 1
    Hide();
    ShowMode();
    jumpval = 0;
    jumphide = jumpmode = false;
    if(!visible)
	{
	    return;
	}
    jumphide = true;
    ConvertJump(0);
#else
    jumpval = 0;
    jumphide = jumpmode = false;
    if(!visible)
	{
	    ShowTimed();
	    if(!visible)
		return;
	    jumphide = true;
	}
#endif
    JumpDisplay();
    jumpactive = true;
}

void cImageControl::PictureZoom(void)
{
    int newzoomval;

    // Center-Parameter kontrollieren
    if(zoom_xcenter < 0)
	zoom_xcenter = 0;
    if(zoom_xcenter >= zoom_xres)
	zoom_xcenter = zoom_xres - 1;
    if(zoom_ycenter < 0)
	zoom_ycenter = 0;
    if(zoom_ycenter >= zoom_yres)
	zoom_ycenter = zoom_yres - 1;


    // Position links oben berechnen
    zoom_xoff = zoom_xcenter - ((zoom_xres / zoom_val) / 2);
    if((zoom_xoff + (zoom_xres / zoom_val)) > zoom_xres)
	{
	    zoom_xoff = zoom_xres - (zoom_xres / zoom_val);
	    zoom_xcenter = zoom_xoff + ((zoom_xres / zoom_val) / 2);
    } else if(zoom_xoff < 0)
	{
	    zoom_xoff = 0;
	    zoom_xcenter = zoom_xoff + ((zoom_xres / zoom_val) / 2);
	}

    zoom_yoff = zoom_ycenter - ((zoom_yres / zoom_val) / 2);
    if((zoom_yoff + (zoom_yres / zoom_val)) > zoom_yres)
	{
	    zoom_yoff = zoom_yres - zoom_yres / zoom_val;
	    zoom_ycenter = zoom_yoff + ((zoom_yres / zoom_val) / 2);
    } else if(zoom_yoff < 0)
	{
	    zoom_yoff = 0;
	    zoom_ycenter = zoom_yoff + ((zoom_yres / zoom_val) / 2);
	}

    newzoomval = 100 / zoom_val;

    dsyslog("zoom_xres   : %d", zoom_xres);
    dsyslog("zoom_yres   : %d", zoom_yres);
    dsyslog("zoom_xcenter: %d", zoom_xcenter);
    dsyslog("zoom_ycenter: %d", zoom_ycenter);
    dsyslog("zoom_xoff   : %d", zoom_xoff);
    dsyslog("zoom_yoff   : %d", zoom_yoff);
    dsyslog("zoom_command: %s", zoom_command);

    ConvertZoom(filename, zoom_command,
		zoom_xres / zoom_val,
		zoom_yres / zoom_val, zoom_xoff, zoom_yoff);
}


void cImageControl::ZoomProcess(eKeys Key)
{
    bool rezoom = false;
    switch (Key)
	{
	case kOk:		// Zoom-in
	case k5:		// Zoom-in
	    if(zoom_val < 10)
		{
		    zoom_val++;
		    PictureZoom();
		}
	    break;
	case kBack:		// Zoom-out
	    if(zoom_val > 2)
		{
		    zoom_val--;
		    PictureZoom();
	    } else
		{
		    zoom_val--;
		    PictureZoom();
		    zoomactive = false;
		    ss = lastSlideMode;
		}
	    break;
	case kLeft:
	case kRight:
	case kUp:
	case kDown:
	    if(((Key == kRight) && !strcmp(zoom_command, "Right"))
	       || ((Key == kLeft) && !strcmp(zoom_command, "Left"))
	       || ((Key == kUp) && strcmp(zoom_command, "Left")
		   && strcmp(zoom_command, "Right")))
		{
		    if(zoom_yoff > 0)
			{
			    dsyslog("kUp: zoom_ycenter =%d+%d", zoom_ycenter,
				    (zoom_yres / zoom_val) / 5);
			    zoom_ycenter -= ((zoom_yres / zoom_val) / 5);
			    dsyslog("zoom_ycenter=%d", zoom_ycenter);
			    rezoom = true;
			}
		}
	    if(((Key == kLeft) && !strcmp(zoom_command, "Right"))
	       || ((Key == kRight) && !strcmp(zoom_command, "Left"))
	       || ((Key == kDown) && strcmp(zoom_command, "Left")
		   && strcmp(zoom_command, "Right")))
		{
		    if(zoom_yoff < (zoom_yres - (zoom_yres / zoom_val)))
			{
			    dsyslog("kDown: zoom_ycenter =%d-%d", zoom_ycenter,
				    (zoom_yres / zoom_val) / 5);
			    zoom_ycenter += ((zoom_yres / zoom_val) / 5);
			    dsyslog("zoom_ycenter=%d", zoom_ycenter);
			    rezoom = true;
			}
		}
	    if(((Key == kUp) && !strcmp(zoom_command, "Right"))
	       || ((Key == kDown) && !strcmp(zoom_command, "Left"))
	       || ((Key == kLeft) && strcmp(zoom_command, "Left")
		   && strcmp(zoom_command, "Right")))
		{
		    if(zoom_xoff > 0)
			{
			    dsyslog("Left: zoom_xcenter=%d-%d", zoom_xcenter,
				    (zoom_xres / zoom_val) / 5);
			    zoom_xcenter -= ((zoom_xres / zoom_val) / 5);
			    dsyslog("zoom_xcenter=%d", zoom_xcenter);
			    rezoom = true;
			}
		}
	    if(((Key == kDown) && !strcmp(zoom_command, "Right"))
	       || ((Key == kUp) && !strcmp(zoom_command, "Left"))
	       || ((Key == kRight) && strcmp(zoom_command, "Left")
		   && strcmp(zoom_command, "Right")))
		{
		    if(zoom_xoff < (zoom_xres - (zoom_xres / zoom_val)))
			{
			    dsyslog("kRight: zoom_xcenter=%d+%d", zoom_xcenter,
				    (zoom_xres / zoom_val) / 5);
			    zoom_xcenter += ((zoom_xres / zoom_val) / 5);
			    dsyslog("zoom_xcenter=%d", zoom_xcenter);
			    rezoom = true;
			}
		}
	    if(rezoom)
		PictureZoom();
	    break;
	default:
	    zoom_val = 1;
	    PictureZoom();
	    zoomactive = false;
	    ss = lastSlideMode;
	    break;
	}

//  if(!zoomactive) {
    //  else DisplayAtBottom(0);
    //}
}

void cImageControl::Zoom(void)
{
    char *filenameloc;
    FILE *pfile;
    char buf[80];

    if(!filename)
	return;


    filenameloc =
	(char *)calloc(1, strlen(filename) + strlen(ImageSetup.TempDir) + 30);
    if(!filenameloc)
	return;

    strcpy(zoom_command, "ORIGINAL");

    zoom_val = 2;
    zoom_xres = 640;
    zoom_yres = 480;

    strcpy(filenameloc, ImageSetup.TempDir);
    strcat(filenameloc, "/");
    strcat(filenameloc, filename);
    strcat(filenameloc, ".cnv");
    if((pfile = fopen(filenameloc, "rt")))
	{
	    dsyslog("open file %s", filenameloc);
	    fgets(buf, sizeof(buf) - 1, pfile);
	    dsyslog("line=%s", buf);
	    sscanf(buf, "%s", zoom_command);
	    fclose(pfile);
    } else
	{
	    dsyslog("error by open file %s", filenameloc);
	}
    dsyslog("zoom_command: %s", zoom_command);

    strcpy(filenameloc, ImageSetup.TempDir);
    strcat(filenameloc, "/");
    strcat(filenameloc, filename);
    strcat(filenameloc, ".par");
    if((pfile = fopen(filenameloc, "rt")))
	{
	    dsyslog("open file %s", filenameloc);

	    fgets(buf, sizeof(buf) - 1, pfile);
	    dsyslog("line=%s", buf);
	    sscanf(buf, "%d", &zoom_xres);

	    fgets(buf, sizeof(buf) - 1, pfile);
	    dsyslog("line=%s", buf);
	    sscanf(buf, "%d", &zoom_yres);

	    fclose(pfile);
    } else
	{
	    dsyslog("error by open file %s", filenameloc);
	}

    free(filenameloc);

    // Standardparameter setzen
    zoom_xcenter = zoom_xres / 2;
    zoom_ycenter = zoom_yres / 2;
    zoom_xoff = zoom_xcenter - ((zoom_xres / zoom_val) / 2);
    if((zoom_xoff + (zoom_xres / zoom_val)) > zoom_xres)
	zoom_xoff = zoom_xres - (zoom_xres / zoom_val);
    else if(zoom_xoff < 0)
	zoom_xoff = 0;

    zoom_yoff = zoom_ycenter - ((zoom_yres / zoom_val) / 2);
    if((zoom_yoff + (zoom_yres / zoom_val)) > zoom_yres)
	zoom_yoff = zoom_yres - (zoom_yres / zoom_val);
    else if(zoom_yoff < 0)
	zoom_yoff = 0;

    if(visible)
	{
	    Hide();
	}
    if(!visible)
	{
	    ShowMode();
	    if(!visible)
		return;
	}

    PictureZoom();

    zoomactive = true;
}

eOSState cImageControl::ProcessKey(eKeys Key)
{
    if(!Active())
	return osEnd;
    if(visible)
	{
	    if(timeoutShow && time(0) > timeoutShow)
		{
		    Hide();
		    ShowMode();
		    timeoutShow = 0;
	    } else
		{
		    if(modeOnly)
			ShowMode();
		    else
			ShowProgress();
		}
	}

    if(zoomactive && Key != kNone)
	{
	    ZoomProcess(Key);
	    return osContinue;
	}

    if(jumpactive && Key != kNone)
	{
	    JumpProcess(Key);
	    return osContinue;
	}

    if(ImageSetup.RemoteLayout == 1)
	{			// Fernbedienungs-Layout: Fast wie bisher, Grn -5, Gelb +5
	    DoShowMode = true;
	    switch (Key)
		{
		case k7:
		    DecSlideTime();
		    break;
		case k9:
		    IncSlideTime();
		    break;

//         case k5:                                     Interface->Error("Zooming not supported yet"); break;
		case k5:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Zoom();
			    if(!zoomactive)
				ss = lastSlideMode;
			}
		    break;

		case kGreen:
		case kGreen | k_Repeat:
		    if(!cstarted)
			PrevImage(5);
		    break;

		case kYellow:
		case kYellow | k_Repeat:
		    if(!cstarted)
			NextImage(5);
		    break;

		case kLeft:
		    if(!cstarted)
			LFlipImage();
		    break;
		case k0:
		    if(!cstarted)
			OriginalImage();
		    break;
		case kRight:
		    if(!cstarted)
			RFlipImage();
		    break;

		case kUp:
		    if(!cstarted)
			NextImage(1);
		    break;
		case kDown:
		    if(!cstarted)
			PrevImage(1);
		    break;

		case kPlay:
		case kPause:
		    if(!cstarted)
			ss = !ss;
		    break;

		case kRed:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Jump();
			    if(!jumpactive)
				ss = lastSlideMode;
			}
		    break;
		case kStop:
		case kBlue:
		case kBack:
		    if(!cstarted)
			{
			    Hide();
			    Stop();
			    return osEnd;
			}
		    // fall trough !!!
		default:
		    DoShowMode = false;
		    switch (Key)
			{
			case kOk:
			    if(visible && !modeOnly)
				{
				    Hide();
				    DoShowMode = true;
			    } else if(modeOnly)
				{
				    Hide();
				    DoShowMode = false;
			    } else
				ShowTimed();
			    break;
			default:
			    break;
			}
		}
	    if(DoShowMode)
		ShowMode();
    } else if(ImageSetup.RemoteLayout == 2)
	{			// Fernbedienungs-Layout: Kreuz zum Drehen, grn -, gelb +
	    DoShowMode = true;
	    switch (Key)
		{
		case k7:
		    DecSlideTime();
		    break;
		case k9:
		    IncSlideTime();
		    break;

//         case k5:                                     Interface->Error("Zoom not supported yet"); break;
		case k0:
		case k5:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Zoom();
			    if(!zoomactive)
				ss = lastSlideMode;
			}
		    break;


		case kGreen:
		    if(!cstarted)
			PrevImage(1);
		    break;
		case kGreen | k_Repeat:
		    if(!cstarted)
			PrevImage(5);
		    break;

		case kYellow:
		    if(!cstarted)
			NextImage(1);
		    break;
		case kYellow | k_Repeat:
		    if(!cstarted)
			NextImage(5);
		    break;

		case kLeft:
		    if(!cstarted)
			LFlipImage();
		    break;
		case kUp:
		    if(!cstarted)
			OriginalImage();
		    break;
		case kRight:
		    if(!cstarted)
			RFlipImage();
		    break;

		case kDown:
		case kPlay:
		case kPause:
		    if(!cstarted)
			ss = !ss;
		    break;

		case kRed:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Jump();
			    if(!jumpactive)
				ss = lastSlideMode;
			}
		    break;
		case kStop:
		case kBlue:
		case kBack:
		    if(!cstarted)
			{
			    Hide();
			    Stop();
			    return osEnd;
			}
		    // fall trough !!!
		default:
		    DoShowMode = false;
		    switch (Key)
			{
			case kOk:
			    if(visible && !modeOnly)
				{
				    Hide();
				    DoShowMode = true;
			    } else if(modeOnly)
				{
				    Hide();
				    DoShowMode = false;
			    } else
				ShowTimed();
			    break;
			default:
			    break;
			}
		}
	    if(DoShowMode)
		ShowMode();
    } else if(ImageSetup.RemoteLayout == 3)
	{			// Fernbedienungs-Layout: Kreuz navigieren, grn -, gelb +
	    DoShowMode = true;
	    switch (Key)
		{
		case k7:
		    DecSlideTime();
		    break;
		case k9:
		    IncSlideTime();
		    break;

		case kGreen:
		    if(!cstarted)
			PrevImage(1);
		    break;
		case kGreen | k_Repeat:
		    if(!cstarted)
			PrevImage(5);
		    break;

		case kYellow:
		    if(!cstarted)
			NextImage(1);
		    break;
		case kYellow | k_Repeat:
		    if(!cstarted)
			NextImage(5);
		    break;

		case kLeft:
		    if(!cstarted)
			PrevImage(1);
		    break;
		case kRight:
		    if(!cstarted)
			NextImage(1);
		    break;

		case k4:
		    if(!cstarted)
			LFlipImage();
		    break;
		case k2:
		    if(!cstarted)
			OriginalImage();
		    break;
		case k6:
		    if(!cstarted)
			RFlipImage();
		    break;

		case kUp:
//         case k5:                                     Interface->Error("Zooming not supported yet"); break;
		case k0:
		case k5:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Zoom();
			    if(!zoomactive)
				ss = lastSlideMode;
			}
		    break;
		case kDown:
		case kPlay:
		case kPause:
		    if(!cstarted)
			ss = !ss;
		    break;

		case kRed:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Jump();
			    if(!jumpactive)
				ss = lastSlideMode;
			}
		    break;
		case kStop:
		case kBlue:
		case kBack:
		    if(!cstarted)
			{
			    Hide();
			    Stop();
			    return osEnd;
			}
		    // fall trough !!!
		default:
		    DoShowMode = false;
		    switch (Key)
			{
			case kOk:
			    if(visible && !modeOnly)
				{
				    Hide();
				    DoShowMode = true;
			    } else if(modeOnly)
				{
				    Hide();
				    DoShowMode = false;
			    } else
				ShowTimed();
			    break;
			default:
			    break;
			}
		}
	    if(DoShowMode)
		ShowMode();
    } else
	{			// Original-Fernbedienungs-Layout plus Springen
	    DoShowMode = true;
	    switch (Key)
		{
		case k0:
		    if(!cstarted)
			OriginalImage();
		    break;
		case kGreen:
		    if(!cstarted)
			ss = !ss;
		    break;
		case kUp:
		    if(!cstarted)
			NextImage(1);
		    break;
		case kDown:
		    if(!cstarted)
			PrevImage(1);
		    break;
		case kRight:
		    if(!cstarted)
			RFlipImage();
		    break;
		case kLeft:
		    if(!cstarted)
			LFlipImage();
		    break;
		case kRed:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Jump();
			    if(!jumpactive)
				ss = lastSlideMode;
			}
		    break;
//         case k5:                                     Interface->Error("Zooming not supported yet"); break;
		case k5:
		    if(!cstarted)
			{
			    lastSlideMode = ss;
			    ss = 0;
			    Zoom();
			    if(!zoomactive)
				ss = lastSlideMode;
			}
		    break;

		case kStop:
		case kBlue:
		case kBack:
		    if(!cstarted)
			{
			    Hide();
			    Stop();
			    return osEnd;
			}
		    // fall trough !!!
		default:
		    DoShowMode = false;
		    switch (Key)
			{
			case kOk:
			    if(visible && !modeOnly)
				{
				    Hide();
				    DoShowMode = true;
			    } else if(modeOnly)
				{
				    Hide();
				    DoShowMode = false;
			    } else
				ShowTimed();
			    break;
			default:
			    break;
			}
		}
	    if(DoShowMode)
		ShowMode();
	}
    return osContinue;
}



// --- cMenuImageBrowse ---------------------------------------------------------

cMenuImageBrowse::cMenuImageBrowse(void):cMenuBrowse(ImageSources.GetSource(), true,
	    tr("Image browser"))
{
    sourcing = false;
    SetButtons();
}

void cMenuImageBrowse::SetButtons(void)
{
    SetHelp(tr("Play"), currentdir ? tr("Parent") : 0, tr("Source"), 0);
    Display();
}

eOSState cMenuImageBrowse::Source(bool second)
{
    if(HasSubMenu())
	return osContinue;

    if(!second)
	{
	    sourcing = true;
	    return AddSubMenu(new
			      cMenuSource(&ImageSources, tr("Image source")));
	}
    sourcing = false;
    cFileSource *src = cMenuSource::GetSelected();
    if(src)
	{
	    ImageSources.SetSource(src);
	    SetSource(src);
	    NewDir(0);
	}
    return osContinue;
}

eOSState cMenuImageBrowse::ProcessKey(eKeys Key)
{
    eOSState state = cMenuBrowse::ProcessKey(Key);

    if(!HasSubMenu() && state == osContinue)
	{			// eval the return value from submenus
	    if(sourcing)
		return Source(true);
	}

    if(state == osBack && lastselect)
	{
	    char *name = lastselect->Path();
	    char *full = source->BuildName(name);
	    cDirItem *item = cMenuBrowse::GetSelected();
	    if(item)
		{
		    Interface->Status(tr("Building SlideShow..."));
		    Interface->Flush();
		    cISlideShow *newss = new cISlideShow(item);
		    if(newss->Load() && newss->Count())
			{
			    lastselect = false;
			    cImageControl::SetSlideShow(newss);
			    free(full);
			    free(name);
			    return osEnd;
		    } else
			{
			    Interface->Error(tr("No Files!"));
			    lastselect = false;
			    return osContinue;
			}
		    delete newss;
		    Interface->Error(tr("Error building SlideShow!"));
		}
	}
    if(state == osUnknown && Key == kYellow)
	return Source(false);
    return state;
}
